@@ -26,6 +26,7 @@
 #include "gva-favorites.h"
 #include "gva-game-store.h"
 #include "gva-main.h"
+#include "gva-mame.h"
 #include "gva-preferences.h"
 #include "gva-ui.h"
 #include "gva-util.h"
@@ -33,10 +34,68 @@
 #define SQL_SELECT_GAMES \
         "SELECT %s FROM available"
 
+void
+tree_view_add_search_expression (GString *expression)
+{
+        gchar *column_name = NULL;
+        gchar *search_text = NULL;
+
+        if (gva_main_get_last_selected_match (&column_name, &search_text))
+                g_string_append_printf (
+                        expression, "%s == '%s'", column_name, search_text);
+        else
+        {
+                search_text = gva_main_get_last_search_text ();
+                if (search_text != NULL && *search_text != '\0')
+                        g_string_append_printf (
+                                expression,
+                                "(name LIKE '%s' OR "
+                                "bios MATCH '%s' OR "
+                                "category MATCH '%s' OR "
+                                "sourcefile LIKE '%s' OR "
+                                "description MATCH '%s' OR "
+                                "manufacturer MATCH '%s' OR "
+                                "year LIKE '%s')",
+                                search_text, search_text,
+                                search_text, search_text,
+                                search_text, search_text,
+                                search_text);
+                else
+                        g_string_append (expression, "name ISNULL");
+        }
+
+        g_free (column_name);
+        g_free (search_text);
+}
+
 static gboolean
-tree_view_show_popup_menu (GdkEventButton *event)
+tree_view_show_popup_menu (GdkEventButton *event,
+                           GtkTreeViewColumn *column)
 {
         GtkMenu *menu;
+        GtkWidget *view;
+        const gchar *column_name;
+        const gchar *column_title;
+        GvaGameStoreColumn column_id;
+        gchar *label, *tooltip;
+
+        view = gtk_tree_view_column_get_tree_view (column);
+        g_object_set_data (G_OBJECT (view), "popup-menu-column", column);
+        column_name = g_object_get_data (G_OBJECT (column), "name");
+        gva_columns_lookup_id (column_name, &column_id);
+        column_title = gva_columns_lookup_title (column_id);
+
+        /* Update the "Remove Column" item in the popup menu. */
+        label = g_strdup_printf (
+                _("Remove %s Column"), column_title);
+        tooltip = g_strdup_printf (
+                _("Remove the \"%s\" column from the game list"),
+                column_title);
+        g_object_set (
+                GVA_ACTION_REMOVE_COLUMN, "label",
+                label, "tooltip", tooltip, NULL);
+        g_free (tooltip);
+        g_free (label);
 
         menu = GTK_MENU (gva_ui_get_managed_widget ("/game-popup"));
 
@@ -146,6 +205,8 @@ gva_tree_view_init (void)
                 G_CALLBACK (tree_view_selection_changed_cb), NULL);
 
         gva_columns_load (view);
+        gva_ui_add_column_actions (view);
+        gva_tree_view_update_status_bar ();
 
         gtk_tree_view_set_search_equal_func (
                 view, (GtkTreeViewSearchEqualFunc)
@@ -186,11 +247,11 @@ gva_tree_view_lookup (const gchar *game)
 gboolean
 gva_tree_view_update (GError **error)
 {
-        GString *expr;
+        GString *expression;
         const gchar *name;
         gboolean success;
 
-        expr = g_string_sized_new (128);
+        expression = g_string_sized_new (128);
 
         switch (gva_tree_view_get_selected_view ())
         {
@@ -198,30 +259,12 @@ gva_tree_view_update (GError **error)
                         break;
 
                 case 1:  /* Favorite Games */
-                        g_string_append (expr, "favorite == \"yes\"");
+                        g_string_append (expression, "favorite == \"yes\"");
                         break;
 
                 case 2:  /* Search Results */
-                {
-                        gchar *text;
-
-                        text = gva_main_get_last_search ();
-                        if (text != NULL && *text != '\0')
-                                g_string_append_printf (
-                                        expr, "(name LIKE '%s' OR "
-					"bios MATCH '%s' OR "
-                                        "category MATCH '%s' OR "
-                                        "sourcefile LIKE '%s' OR "
-                                        "description MATCH '%s' OR "
-                                        "manufacturer MATCH '%s' OR "
-                                        "year LIKE '%s')",
-                                        text, text, text, text,
-                                        text, text, text);
-                        else
-                                g_string_append (expr, "name ISNULL");
-                        g_free (text);
+                        tree_view_add_search_expression (expression);
                         break;
-                }
 
                 default:
                         g_assert_not_reached ();
@@ -229,13 +272,13 @@ gva_tree_view_update (GError **error)
 
         if (!gva_preferences_get_show_clones ())
         {
-                if (expr->len > 0)
-                        g_string_append (expr, " AND ");
-                g_string_append (expr, "cloneof ISNULL");
+                if (expression->len > 0)
+                        g_string_append (expression, " AND ");
+                g_string_append (expression, "cloneof ISNULL");
         }
 
-        success = gva_tree_view_run_query (expr->str, error);
-        g_string_free (expr, TRUE);
+        success = gva_tree_view_run_query (expression->str, error);
+        g_string_free (expression, TRUE);
 
         if (!success)
                 return FALSE;
@@ -249,7 +292,7 @@ gva_tree_view_update (GError **error)
 
 /**
  * gva_tree_view_run_query:
- * @expr: an SQL "where" expression
+ * @expression: an SQL "where" expression
  * @error: return location for a #GError, or %NULL
  *
  * Similar to gva_tree_view_update() but applies custom criteria to the game
@@ -258,7 +301,7 @@ gva_tree_view_update (GError **error)
  * Returns: %TRUE on success, %FALSE if an error occurred
  **/
 gboolean
-gva_tree_view_run_query (const gchar *expr,
+gva_tree_view_run_query (const gchar *expression,
                          GError **error)
 {
         GvaGameStoreColumn column_id;
@@ -292,8 +335,8 @@ gva_tree_view_run_query (const gchar *expr,
         g_string_printf (string, SQL_SELECT_GAMES, columns);
         g_free (columns);
 
-        if (expr != NULL && *expr != '\0')
-                g_string_append_printf (string, " WHERE %s", expr);
+        if (expression != NULL && *expression != '\0')
+                g_string_append_printf (string, " WHERE %s", expression);
 
         window = gtk_widget_get_parent_window (GTK_WIDGET (view));
         display = gtk_widget_get_display (GTK_WIDGET (view));
@@ -329,6 +372,7 @@ gva_tree_view_run_query (const gchar *expr,
 
         gtk_tree_view_set_model (view, model);
         gtk_tree_view_columns_autosize (view);
+        gva_tree_view_update_status_bar ();
         g_object_unref (model);
 
         /* Need to reset the search column after loading a new model,
@@ -360,6 +404,83 @@ gva_tree_view_get_model (void)
         return gtk_tree_view_get_model (view);
 }
 
+/**
+ * gva_tree_view_update_status_bar:
+ *
+ * Puts a message in the main status bar containing the MAME version and the
+ * number of games displayed in the current view.  This message gets shown
+ * when the status bar has nothing more important to show.
+ **/
+void
+gva_tree_view_update_status_bar (void)
+{
+        GtkTreeView *view;
+        GtkTreeModel *model;
+        GString *message;
+        const gchar *units;
+        gchar *mame_version;
+        guint context_id;
+        gint count = 0;
+        GError *error = NULL;
+
+        view = GTK_TREE_VIEW (GVA_WIDGET_MAIN_TREE_VIEW);
+        model = gtk_tree_view_get_model (view);
+        message = g_string_sized_new (128);
+
+        mame_version = gva_mame_get_version (&error);
+        gva_error_handle (&error);
+
+        if (model != NULL)
+                count = gtk_tree_model_iter_n_children (model, NULL);
+
+        switch (gva_tree_view_get_selected_view ())
+        {
+                case 0:
+                        units = ngettext (
+                                "Available Game",
+                                "Available Games",
+                                count);
+                        break;
+
+                case 1:
+                        units = ngettext (
+                                "Favorite Game",
+                                "Favorite Games",
+                                count);
+                        break;
+
+                case 2:
+                        units = ngettext (
+                                "Search Result",
+                                "Search Results",
+                                count);
+                        break;
+
+                default:
+                        g_return_if_reached ();
+        }
+
+        if (mame_version != NULL)
+        {
+                g_string_append (message, mame_version);
+                g_free (mame_version);
+
+                if (count > 0)
+                {
+                        /* Append a UTF-8 encoded bullet. */
+                        g_string_append (message, "  •  ");
+                }
+        }
+
+        if (count > 0)
+                g_string_append_printf (message, "%d %s", count, units);
+
+        context_id = gva_main_statusbar_get_context_id (G_STRFUNC);
+        gva_main_statusbar_pop (context_id);
+        gva_main_statusbar_push (context_id, message->str);
+        g_string_free (message, TRUE);
+}
+
 /**
  * gva_tree_view_get_selected_game:
  *
@@ -618,19 +739,21 @@ gva_tree_view_button_press_event_cb (GtkTreeView *view,
 {
         if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
         {
+                GtkTreeViewColumn *column;
                 GtkTreePath *path;
                 gboolean valid;
 
                 /* Select the row that was clicked. */
                 valid = gtk_tree_view_get_path_at_pos (
-                        view, event->x, event->y, &path, NULL, NULL, NULL);
+                        view, event->x, event->y, &path,
+                        &column, NULL, NULL);
                 if (valid)
                 {
                         gtk_tree_view_set_cursor (view, path, NULL, FALSE);
                         gtk_widget_grab_focus (GTK_WIDGET (view));
                         gtk_tree_path_free (path);
 
-                        return tree_view_show_popup_menu (event);
+                        return tree_view_show_popup_menu (event, column);
                 }
         }
 
@@ -650,7 +773,11 @@ gva_tree_view_button_press_event_cb (GtkTreeView *view,
 gboolean
 gva_tree_view_popup_menu_cb (GtkTreeView *view)
 {
-        return tree_view_show_popup_menu (NULL);
+        GtkTreeViewColumn *column;
+
+        gtk_tree_view_get_cursor (view, NULL, &column);
+
+        return tree_view_show_popup_menu (NULL, column);
 }
 
 /**
